@@ -1,6 +1,6 @@
 #######################################################################
 # stream -  Infrastructure for Data Stream Mining
-# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest 
+# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,9 +31,9 @@
 DSD <- function(...) stop("DSD is an abstract class and cannot be instantiated!")
 DSD_R <- function(...) stop("DSD_R is an abstract class and cannot be instantiated!")
 
-get_points <- function(x, n=1, outofpoints=c("stop", "warn", "ignore"), ...) 
+get_points <- function(x, n=1, outofpoints=c("stop", "warn", "ignore"), ...)
   UseMethod("get_points")
-get_points.default <- function(x, n=1, 
+get_points.default <- function(x, n=1,
   outofpoints=c("stop", "warn", "ignore"), ...) {
   stop(gettextf("get_points not implemented for class '%s'.",
     paste(class(x), collapse=", ")))
@@ -52,64 +52,66 @@ reset_stream.DSD <- function(dsd, pos=1) {
 ### helper
 print.DSD <- function(x, ...) {
   .nodots(...)
-  
+
   k <- x[["k"]]
   if(is.null(k)) k <- NA
   d <- x[["d"]]
   if(is.null(d)) d <- NA
-  
+
   cat(.line_break(x$description))
-  cat("Class:", paste(class(x), collapse=", "), "\n") 
+  cat("Class:", paste(class(x), collapse=", "), "\n")
   cat(paste('With', k, 'clusters', 'in', d, 'dimensions', '\n'))
 }
 
 summary.DSD <- function(object, ...) print(object)
 
-plot.DSD <- function(x, n = 500, col= NULL, pch= NULL, 
+plot.DSD <- function(x, n = 500, col= NULL, pch= NULL,
   ..., method="pairs", dim=NULL, alpha=.6) {
   ## method can be pairs, plot or pc (projection with PCA)
-  d <- get_points(x, n, cluster = TRUE)
   
-  ### make sure to plot noise
+  d <- get_points(x, n, cluster = !is.null(x$class))
   assignment <- attr(d, "cluster")
   
   ### stream has no assignments!
   if(length(assignment)==0) assignment <- rep(1L, nrow(d))
-  
+
   noise <- is.na(assignment)
-  
+
   ### assignment is not numeric
   if(!is.numeric(assignment)) assignment <- as.integer(as.factor(assignment))
-  
+
   ### add alpha shading to color
   if(is.null(col)) {
     col <- rgb(cbind(t(col2rgb(assignment)/255)), alpha=alpha)
   }else{
     if(length(col)==1L) col <- rep(col, length(assignment))
   }
-    
+
   col[noise] <-  .noise_col
-  
+
   if(is.null(pch)) {
     #pch <- rep(1, n)
     pch <- as.integer(assignment)
     pch[noise] <- .noise_pch
   }
-  
-  if(!is.null(dim)) d <- d[,dim]
-  
+
+  if(!is.null(dim)) d <- d[,dim, drop = FALSE]
+
   if(ncol(d)>2 && method=="pairs") {
     pairs(d, col=col, pch=pch, ...)
   } else if(ncol(d)>2 && method=="pc") {
     ## we assume Euclidean here
     p <- prcomp(d)
- 
+
     plot(p$x, col=col, pch=pch, ...)
     title(sub = paste("Explains ",
-      round(sum(p$sdev[1:2]) / sum(p$sdev)* 100, 2), 
+      round(sum(p$sdev[1:2]) / sum(p$sdev)* 100, 2),
       "% of the point variability", sep=""))
+  } else if(ncol(d) == 1) {
+    plot(d[[1]], rep(0, length(d[[1]])), col = assignment, pch= pch,
+      ylab = "", xlab = colnames(d)[1], ...)
   } else {
-    if(ncol(d)>2) d <- d[,1:2]
+    if(ncol(d)>2) d <- d[,1:2, drop = FALSE]
     plot(d, col=col, pch=pch, ...)
   }
 }
